local ldb_tip
local f = CreateFrame("Button", "ZoneInfo")
local tourist = LibStub("LibTourist-3.0")
local ldb_obj = LibStub("LibDataBroker-1.1"):NewDataObject("ZoneInfo", {
	icon     = "Interface\\Icons\\INV_Misc_Book_11.png",
	label    = "ZoneInfo"
})

ZoneInfoSV = ZoneInfoSV or { full = true, range = true, drag = false }

local function get_zone_info(zone)
	local continent            = zone and (tourist:GetContinent(zone) or "Unknown")
	zone                       = zone or GetRealZoneText()
	local min_level, max_level = tourist:GetLevel(zone)
	min_level                  = min_level == 0 and 1 or min_level
	max_level                  = max_level == 0 and (select(4, GetBuildInfo()) >= 40000 and 85 or 80) or max_level
	local red, green, blue     = tourist:GetLevelColor(zone)
	local color                = ("|cff%02x%02x%02x"):format(red * 255, green * 255, blue * 255)
	local range                = ("%2d-%2d"):format(min_level, max_level)

	return color..range.."|r", continent or zone
end

local function iterate_zones(tip, which)
	local _, zones = tourist["IterateRecommended"..which](tourist)

	if type(zones) == "table" then
		tip:AddLine("|cff8888ee Appropriate "..which.." for level range:|r")

		for zone in pairs(zones) do
			local range, continent = get_zone_info(zone)
			local complex          = tourist:GetComplex(zone)
			zone                   = which == "Instances" and (complex and complex.." " or "")..zone.."  ("..(tourist:GetInstanceZone(zone) or "Unknown")..")" or zone

			tip:AddDoubleLine(range.."|cffeeeeee - "..zone.."|r", "|cffeeeeee"..continent.."|r")
		end
	end
end

local function set_ldb_obj_text()
	local range, zone = get_zone_info()
	local minimap     = GetMinimapZoneText()
	zone              = ZoneInfoSV.full and (minimap and zone.." ("..minimap..")" or zone) or (minimap or zone)
	ldb_obj.text      = zone.."|r"..(ZoneInfoSV.range and " ["..range.."]" or "")
end

function ldb_obj.OnTooltipShow(tip)
	if not ldb_tip then ldb_tip = tip end

	tip:ClearLines()

	tip:AddLine("ZoneInfo")
	tip:AddLine(" ")

	if ZoneInfoSV.drag then
		tip:AddLine("|cff69b950Control + Click:|r |cffeeeeeeToggles plugin-dragging mode.|r")
		tip:Show()
		return
	end


       local range, zone = get_zone_info()
 
        tip:AddLine("You are in " ..zone.."|r")

        local min2Fish = tourist:GetFishingLevel(zone)
        
        if min2Fish then
           tip:AddLine("Fishing best at skill level " ..min2Fish)
        end

        tip:AddLine("|r")


	local _, borders = tourist:IterateBorderZones(GetRealZoneText())


	if type(borders) == "table" then
		tip:AddLine("|cff8888eeZones bordering this one:|r")

		for zone in pairs(borders) do
			local range, continent = get_zone_info(zone)
	
		tip:AddDoubleLine(range.."|cffeeeeee - "..zone.."|r", "|cffeeeeee"..continent.."|r")
		end
	
		tip:AddLine(" ")
	end

	iterate_zones(tip, "Instances")
	-- iterate_zones(tip, "BattleGrounds")
	tip:AddLine(" ")
	iterate_zones(tip, "Zones")

	local range, zone = get_zone_info()

	tip:AddLine(" ")
	tip:AddDoubleLine("|cffffd700The level range of "..zone.." is:|r", range)
	tip:AddLine(" ")
	tip:AddLine("|cff69b950Click:|r |cffeeeeeeToggles the World Map open/closed.|r")
	tip:AddLine("|cff69b950Alt + Click:|r |cffeeeeeeToggles the zone/sub-zone display style.|r")
	tip:AddLine("|cff69b950Shift + Click:|r |cffeeeeeeToggles showing the range on the display.|r")
	tip:AddLine("|cff69b950Control + Click:|r |cffeeeeeeToggles plugin-dragging mode.|r")


	tip:Show()
end

function ldb_obj.OnClick()
	if IsControlKeyDown() then
		ZoneInfoSV.drag = not ZoneInfoSV.drag

		ldb_obj.OnTooltipShow(ldb_tip)

	elseif IsAltKeyDown() and not ZoneInfoSV.drag then
		ZoneInfoSV.full = not ZoneInfoSV.full

		set_ldb_obj_text()

	elseif IsShiftKeyDown() and not ZoneInfoSV.drag then
		ZoneInfoSV.range = not ZoneInfoSV.range

		set_ldb_obj_text()

	elseif not ZoneInfoSV.drag then
		local func = ToggleWorldMap or ToggleFrame		
		func(WorldMapFrame)
	end
end

--  f:RegisterEvent"PLAYER_LOGIN"
-- f:RegisterEvent"PLAYER_ALIVE"
f:RegisterEvent"ZONE_CHANGED"
f:RegisterEvent"ZONE_CHANGED_INDOORS"
f:RegisterEvent"ZONE_CHANGED_NEW_AREA"

f:SetScript("OnEvent", set_ldb_obj_text)
